// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.generate;

import java.awt.GridBagConstraints;
import static java.awt.GridBagConstraints.BOTH;
import static java.awt.GridBagConstraints.CENTER;
import java.awt.Insets;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import static javax.swing.BorderFactory.createEtchedBorder;
import static javax.swing.BorderFactory.createTitledBorder;
import javax.swing.JPanel;

import com.kiwisoft.generate.model.UserParameter;
import com.kiwisoft.generate.model.ParameterBean;
import com.kiwisoft.generate.model.UserType;
import com.kiwisoft.utils.ObservableDynaBean;

/**
 * A panel for properties with a list of user type elements.
 *
 * @author Stefan Stiller
 */
class GroupListPanel extends TitledParameterPanel
{
	private ParameterBean bean;
	private Map<ParameterBean, GroupPanel> components=new HashMap<ParameterBean, GroupPanel>();
	private int row;

	public GroupListPanel(ParameterBean bean, UserParameter parameter)
	{
		super(parameter);
		this.bean=bean;
		List<ParameterBean> values=(List<ParameterBean>)this.bean.get(getParameter().getName());
		for (ParameterBean value : values) addComponent(value);
	}

	protected boolean canAddElement()
	{
		return true;
	}

	protected void addElement()
	{
		List<Object> list=(List<Object>)bean.get(getParameter().getName());
		UserType userType=(UserType)getParameter().getType();
		try
		{
			ParameterBean newBean=(ParameterBean)userType.createDynaClass().newInstance();
			list.add(newBean);
			addComponent(newBean);
			getElementsPanel().updateUI();
		}
		catch (Exception e)
		{
			throw new RuntimeException(e);
		}
	}

	private void addComponent(ParameterBean value)
	{
		GroupPanel groupPanel=new GroupPanel(value, (UserType)getParameter().getType(), new RemoveElementAction(value));
		JPanel panel=getElementsPanel();
		components.put(value, groupPanel);
		groupPanel.setBorder(createTitledBorder(createEtchedBorder(), getParameter().getType().getDisplayableName()+" #"+components.size()));
		panel.add(groupPanel, new GridBagConstraints(0, row++, 1, 1, 1.0, 0.0, CENTER, BOTH, new Insets(2, 2, 2, 0), 0, 0));
	}

	protected boolean canRemoveElement(Object value)
	{
		List<Object> list=(List<Object>)bean.get(getParameter().getName());
		return list.contains(value);
	}

	@SuppressWarnings({"SuspiciousMethodCalls"})
	protected void removeElement(Object value)
	{
		GroupPanel component=components.remove(value);
		JPanel panel=getElementsPanel();
		if (component!=null)
		{
			List<ObservableDynaBean> values=(List<ObservableDynaBean>)bean.get(getParameter().getName());
			values.remove(value);
			component.dispose();
			panel.remove(component);
			for (int i=0; i<panel.getComponentCount(); i++)
			{
				GroupPanel groupPanel=(GroupPanel)panel.getComponent(i);
				groupPanel.setBorder(createTitledBorder(createEtchedBorder(), getParameter().getType().getDisplayableName()+" #"+(i+1)));
			}
			panel.updateUI();
		}
	}

	public void dispose()
	{
		for (GroupPanel groupPanel : components.values()) groupPanel.dispose();
		super.dispose();
	}
}
