// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.generate.formats;

import javax.swing.Icon;

import com.intellij.openapi.fileTypes.FileTypeManager;
import com.intellij.openapi.fileTypes.FileType;

import com.kiwisoft.utils.format.DefaultObjectFormat;
import com.kiwisoft.utils.StringUtils;
import com.kiwisoft.generate.model.TemplateFile;

/**
 * Class used for formatting and rendering of {@link TemplateFile} objects.
 *
 * @author Stefan Stiller
 */
public class TemplateFileFormat extends DefaultObjectFormat
{
	public TemplateFileFormat()
	{
		super(DEFAULT);
	}

	public boolean canFormat(Class aClass)
	{
		return aClass!=null && TemplateFile.class.isAssignableFrom(aClass);
	}

	public String format(Object value)
	{
		if (value instanceof TemplateFile)
		{
			String name=((TemplateFile)value).getName();
			if (StringUtils.isEmpty(name)) return "<Unnamed File>";
			return name;
		}
		return super.format(value);
	}

	public Icon getIcon(Object value)
	{
		if (value instanceof TemplateFile)
		{
			TemplateFile file=(TemplateFile)value;
			String name=file.getName();
			if (name!=null)
			{
				FileType fileType=FileTypeManager.getInstance().getFileTypeByFileName(name);
				return fileType.getIcon();
			}
		}
		return super.getIcon(value);
	}
}
