// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.generate.model.ui;

import java.awt.GridBagConstraints;
import static java.awt.GridBagConstraints.*;
import java.awt.Insets;
import java.awt.GridBagLayout;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeSupport;
import javax.swing.*;

import com.kiwisoft.generate.lookups.TypeLookup;
import com.kiwisoft.generate.model.*;
import com.kiwisoft.generate.DefaultTableConfiguration;
import com.kiwisoft.utils.gui.lookup.LookupField;
import com.kiwisoft.utils.gui.adapters.*;
import com.kiwisoft.utils.gui.table.*;
import com.kiwisoft.utils.Disposable;
import com.kiwisoft.utils.ObservableBean;

/**
 * A panel for editing details of a user parameter. 
 *
 * @author Stefan Stiller
 */
public class UserParameterPanel extends DetailsPanel<UserParameter>
{
	private Template template;
	private TypeWrapper typeWrapper;
	private JLabel defaultLabel;
	private GridBagConstraints defaultConstraint;
	private JLabel valuesLabel;
	private GridBagConstraints valuesConstraint;

	public UserParameterPanel(Template template, UserParameter parameter)
	{
		super(parameter);
		this.template=template;
		initComponents();
	}

	private void initComponents()
	{
		JTextField nameField=new JTextField();
		JTextField displayNameField=new JTextField();
		JTextArea descriptionField=new JTextArea();
		descriptionField.setWrapStyleWord(true);
		descriptionField.setLineWrap(true);
		descriptionField.setRows(3);
		JScrollPane descriptionPanel=new JScrollPane(descriptionField);
		LookupField<Type> typeField=new LookupField<Type>(new TypeLookup(template));

		int row=0;
		add(new JLabel("Name:"),
			new GridBagConstraints(0, row, 2, 1, 0.0, 0.0, WEST, NONE, new Insets(5, 5, 0, 5), 0, 0));
		row++;
		add(nameField,
			new GridBagConstraints(0, row, 2, 1, 1.0, 0.0, WEST, HORIZONTAL, new Insets(2, 5, 5, 5), 0, 0));

		row++;
		add(new JLabel("Display Name:"),
			new GridBagConstraints(0, row, 2, 1, 0.0, 0.0, WEST, NONE, new Insets(5, 5, 0, 5), 0, 0));
		row++;
		add(displayNameField,
			new GridBagConstraints(0, row, 2, 1, 1.0, 0.0, WEST, HORIZONTAL, new Insets(2, 5, 5, 5), 0, 0));

		row++;
		add(new JLabel("Description:"),
			new GridBagConstraints(0, row, 2, 1, 0.0, 0.0, WEST, NONE, new Insets(5, 5, 0, 5), 0, 0));
		row++;
		add(descriptionPanel,
			new GridBagConstraints(0, row, 2, 1, 1.0, 0.0, WEST, BOTH, new Insets(2, 5, 5, 5), 0, 0));

		row++;
		add(new JLabel("Type:"),
			new GridBagConstraints(0, row, 1, 1, 0.0, 0.0, WEST, NONE, new Insets(5, 5, 0, 5), 0, 0));
		add(defaultLabel=new JLabel("Default:"),
			new GridBagConstraints(1, row, 1, 1, 0.0, 0.0, WEST, NONE, new Insets(5, 5, 0, 5), 0, 0));
		row++;
		add(typeField,
			new GridBagConstraints(0, row, 1, 1, 0.5, 0.0, WEST, HORIZONTAL, new Insets(2, 5, 5, 5), 0, 0));
		defaultConstraint=new GridBagConstraints(1, row, 1, 1, 0.5, 0.0, WEST, HORIZONTAL, new Insets(2, 5, 5, 5), 0, 0);

		row++;
		add(createFlagPanel(),
			new GridBagConstraints(0, row, 1, 2, 0.5, 1.0, NORTHWEST, NONE, new Insets(5, 5, 0, 5), 0, 0));
		add(valuesLabel=new JLabel("Values:"),
			new GridBagConstraints(1, row, 1, 1, 0.5, 0.0, NORTHWEST, NONE, new Insets(5, 5, 0, 5), 0, 0));

		row++;
		valuesConstraint=new GridBagConstraints(1, row, 1, 1, 0.5, 1.0, CENTER, BOTH, new Insets(2, 5, 5, 5), 0, 0);

		addAdapter(getListenerSupport().bind(nameField, getObject(), "name"));
		addAdapter(getListenerSupport().bind(displayNameField, getObject(), "displayName"));
		addAdapter(getListenerSupport().bind(descriptionField, getObject(), "description"));
		addAdapter(getListenerSupport().bind(typeField, getObject(), "type"));
		updateFields();
		getListenerSupport().installPropertyChangeListener(getObject(), "type", new TypeChangeListener());
	}

	private JPanel createFlagPanel()
	{
		JCheckBox requiredField=new JCheckBox("Required");
		JCheckBox collectionField=new JCheckBox("Collection");
		JCheckBox restrictedField=new JCheckBox("Restricted");

		JPanel panel=new JPanel(new GridBagLayout());
		int row=0;
		panel.add(requiredField,
			new GridBagConstraints(0, row++, 1, 1, 0.0, 0.0, NORTHWEST, NONE, new Insets(5, 5, 5, 5), 0, 0));
		panel.add(restrictedField,
			new GridBagConstraints(0, row++, 1, 1, 0.0, 0.0, NORTHWEST, NONE, new Insets(5, 5, 5, 5), 0, 0));
		panel.add(collectionField,
			new GridBagConstraints(0, row, 1, 1, 0.0, 1.0, NORTHWEST, NONE, new Insets(5, 5, 5, 5), 0, 0));

		addAdapter(getListenerSupport().bind(requiredField, getObject(), "required"));
		addAdapter(getListenerSupport().bind(collectionField, getObject(), "collection"));
		addAdapter(getListenerSupport().bind(restrictedField, getObject(), "restricted"));
		return panel;
	}

	public boolean stopEditing()
	{
		if (!super.stopEditing()) return false;
		if (typeWrapper!=null) return typeWrapper.stopEditing();
		return true;
	}

	private void updateFields()
	{
		valuesLabel.setVisible(false);
		defaultLabel.setVisible(false);
		Type type=getObject().getType();
		try
		{
			if (type instanceof StringType) createStringField();
			else if (type instanceof IntegerType) createIntegerField();
			else if (type instanceof DecimalType) createDoubleField();
			else if (type instanceof BooleanType) createBooleanField();
		}
		catch (Exception e)
		{
			throw new RuntimeException(e);
		}
		if (typeWrapper!=null) getListenerSupport().addDisposable(typeWrapper);
		defaultLabel.setVisible(typeWrapper!=null);
		valuesLabel.setVisible(typeWrapper!=null && typeWrapper.valuesComponent!=null);
		updateUI();
	}

	private void createStringField() throws Exception
	{
		JTextField component=new JTextField();
		TextComponentAdapter adapter=new TextComponentAdapter(component);
		typeWrapper=new TypeWrapper(adapter, component);
		adapter.bind(typeWrapper, "stringValue");
		add(component, defaultConstraint);

		SetTable valuesTable=new SetTable(String.class);
		DefaultTableConfiguration configuration=new DefaultTableConfiguration("parameter.values");
		configuration.setSupportWidths(false);
		valuesTable.initializeColumns(configuration);
		typeWrapper.valuesComponent=new JScrollPane(valuesTable);
		typeWrapper.valuesAdapter=new SetAdapter(valuesTable);
		typeWrapper.valuesAdapter.bind(getObject(), "values");
		add(typeWrapper.valuesComponent, valuesConstraint);
	}

	private void createIntegerField() throws Exception
	{
		JFormattedTextField component=new JFormattedTextField();
		NumberAdapter adapter=new NumberAdapter(component);
		typeWrapper=new TypeWrapper(adapter, component);
		adapter.bind(typeWrapper, "integerValue");
		add(component, defaultConstraint);

//		SetTable valuesTable=new SetTable(Integer.class);
//		typeWrapper.valuesComponent=new JScrollPane(valuesTable);
//		typeWrapper.valuesAdapter=new SetAdapter(valuesTable);
//		typeWrapper.valuesAdapter.bind(getObject(), "values");
//		add(typeWrapper.valuesComponent, valuesConstraint);
	}

	private void createDoubleField() throws Exception
	{
		JFormattedTextField component=new JFormattedTextField();
		NumberAdapter adapter=new NumberAdapter(component);
		typeWrapper=new TypeWrapper(adapter, component);
		adapter.bind(typeWrapper, "doubleValue");
		add(component, defaultConstraint);

//		SetTable valuesTable=new SetTable(Double.class);
//		typeWrapper.valuesComponent=new JScrollPane(valuesTable);
//		typeWrapper.valuesAdapter=new SetAdapter(valuesTable);
//		typeWrapper.valuesAdapter.bind(getObject(), "values");
//		add(typeWrapper.valuesComponent, valuesConstraint);
	}

	private void createBooleanField() throws Exception
	{
		JCheckBox component=new JCheckBox();
		CheckBoxAdapter adapter=new CheckBoxAdapter(component);
		typeWrapper=new TypeWrapper(adapter, component);
		adapter.bind(typeWrapper, "booleanValue");
		add(component, defaultConstraint);
	}

	public String getHelpTopic()
	{
		return "KiwiGenerator.property.details";
	}

	public class TypeWrapper implements Disposable, PropertyChangeListener, ObservableBean
	{
		private JComponent defaultComponent;
		private ComponentAdapter defaultAdapter;
		private JComponent valuesComponent;
		private ComponentAdapter valuesAdapter;

		private PropertyChangeSupport changeSupport=new PropertyChangeSupport(this);

		public TypeWrapper(ComponentAdapter adapter, JComponent component)
		{
			this.defaultAdapter=adapter;
			this.defaultComponent=component;
			getObject().addPropertyChangeListener("defaultValue", this);
		}

		public void dispose()
		{
			getObject().removePropertyChangeListener("defaultValue", this);
			defaultAdapter.unbind();
			remove(defaultComponent);
			if (valuesAdapter!=null) valuesAdapter.unbind();
			if (valuesComponent!=null) remove(valuesComponent);
		}

		public String getStringValue()
		{
			return (String)getObject().getDefaultValue();
		}

		public void setStringValue(String value)
		{
			getObject().setDefaultValue(value);
		}

		public Integer getIntegerValue()
		{
			return (Integer)getObject().getDefaultValue();
		}

		public void setIntegerValue(Integer value)
		{
			getObject().setDefaultValue(value);
		}

		public Double getDoubleValue()
		{
			return (Double)getObject().getDefaultValue();
		}

		public void setDoubleValue(Double value)
		{
			getObject().setDefaultValue(value);
		}

		public boolean isBooleanValue()
		{
			return Boolean.TRUE.equals(getObject().getDefaultValue());
		}

		public void setBooleanValue(boolean value)
		{
			getObject().setDefaultValue(value);
		}

		public void propertyChange(PropertyChangeEvent evt)
		{
			firePropertyChange("value", evt.getOldValue(), evt.getNewValue());
		}

		public void addPropertyChangeListener(PropertyChangeListener listener)
		{
			changeSupport.addPropertyChangeListener(listener);
		}

		public void removePropertyChangeListener(PropertyChangeListener listener)
		{
			changeSupport.removePropertyChangeListener(listener);
		}

		public void addPropertyChangeListener(String propertyName, PropertyChangeListener listener)
		{
			changeSupport.addPropertyChangeListener(propertyName, listener);
		}

		public void removePropertyChangeListener(String propertyName, PropertyChangeListener listener)
		{
			changeSupport.removePropertyChangeListener(propertyName, listener);
		}

		public void firePropertyChange(String propertyName, Object oldValue, Object newValue)
		{
			changeSupport.firePropertyChange(propertyName, oldValue, newValue);
		}

		public boolean stopEditing()
		{
			if (!defaultAdapter.stopEditing()) return false;
			if (valuesAdapter!=null)
			{
				if (!valuesAdapter.stopEditing()) return false;
			}
			return true;
		}
	}

	private class TypeChangeListener implements PropertyChangeListener
	{
		public void propertyChange(PropertyChangeEvent evt)
		{
			getListenerSupport().dispose(typeWrapper);
			typeWrapper=null;
			updateFields();
		}
	}
}
