// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.generate.tree;

import java.awt.Font;
import java.util.Vector;
import javax.swing.JTree;

import com.kiwisoft.generate.model.SystemMethod;
import com.kiwisoft.generate.model.SystemParameter;
import com.kiwisoft.generate.model.Type;
import com.kiwisoft.generate.model.Types;
import com.kiwisoft.utils.gui.tree.GenericTreeNode;
import com.kiwisoft.utils.gui.tree.TreeNodeLeaf;

/**
 * A tree node representing a hardcoded method.
 *
 * @author Stefan Stiller
 */
public class SystemMethodsNode extends GenericTreeNode<String>
{
	public SystemMethodsNode()
	{
		super("Predefined Methods");
	}

	public Vector<GenericTreeNode> loadChildren()
	{
		Vector<GenericTreeNode> nodes=super.loadChildren();
		Type stringType=Types.valueOf("String");
		Type intType=Types.valueOf("Integer");
		nodes.add(new GenericTreeNode<String>("Dates",
											  new TreeNodeLeaf<SystemMethod>(
												  new SystemMethod("date", "Returns the current date formatted with the given pattern.",
																   new SystemParameter("pattern",
																					   "The formatting pattern like supported by java.text.SimpleDateFormat.",
																					   stringType)))));
		nodes.add(new GenericTreeNode<String>("Env",
											  new TreeNodeLeaf<SystemMethod>(
												  new SystemMethod("get",
																   "Returns the given environment variable.",
																   new SystemParameter("variable", "The name of the environment variable.", stringType)))));
		nodes.add(new GenericTreeNode<String>("Strings",
											  new TreeNodeLeaf<SystemMethod>(
												  new SystemMethod("capitalize", "Change the first letter into upper case.",
																   new SystemParameter("value", "The string to be modified.", stringType))),
											  new TreeNodeLeaf<SystemMethod>(
												  new SystemMethod("decapitalize", "Change the first letter into lower case.",
																   new SystemParameter("value", "The string to be modified.", stringType))),
											  new TreeNodeLeaf<SystemMethod>(
												  new SystemMethod("fillLeft",
																   "Prepends the given string to the given baseString until it has at least the given length.",
																   new SystemParameter("baseString", "The starting string.", stringType),
																   new SystemParameter("string", "The string which is prepended to be baseString.", stringType),
																   new SystemParameter("length", "The length of the returned string.", intType))),
											  new TreeNodeLeaf<SystemMethod>(
												  new SystemMethod("fillRight",
																   "Appends the given string to the given baseString until it has at least the given length.",
																   new SystemParameter("baseString", "The starting string.", stringType),
																   new SystemParameter("string", "The string which is appended to the baseString.", stringType),
																   new SystemParameter("length", "The length of the returned string.", intType))),
											  new TreeNodeLeaf<SystemMethod>(
												  new SystemMethod("formatJavaIdentifier",
																   "Formats the given value as a displayable string. 'aJavaProperty' will be changed to 'A Java Property'.",
																   new SystemParameter("value", "The string to be modified.", stringType)))));
		nodes.add(new GenericTreeNode<String>("System",
											  new TreeNodeLeaf<SystemMethod>(
												  new SystemMethod("get", "Returns the given system property.",
																   new SystemParameter("property", "The name of the property.", stringType)))));
		return nodes;
	}

	public Font getFont(JTree tree)
	{
		return tree.getFont().deriveFont(Font.BOLD);
	}
}
