// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.utils;

import java.text.DateFormat;
import java.text.ParseException;
import java.util.Date;
import java.util.Calendar;

/**
 * @author Stefan Stiller
 * @version $Revision: 1.6 $, $Date: 2006/03/24 18:27:21 $
 */
public class DateUtils
{
	public final static long SECOND=1000;
	public final static long MINUTE=60*SECOND;
	public final static long HOUR=60*MINUTE;
	public final static long DAY=24*HOUR;
	public final static long WEEK=7*DAY;

	private static DateFormat[] dateFormats=
			{
				DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT),
				DateFormat.getDateInstance(DateFormat.SHORT),
				DateFormat.getDateTimeInstance(DateFormat.MEDIUM, DateFormat.MEDIUM),
				DateFormat.getDateInstance(DateFormat.MEDIUM),
				DateFormat.getDateInstance(DateFormat.LONG),
				DateFormat.getDateTimeInstance(DateFormat.LONG, DateFormat.LONG),
				DateFormat.getDateInstance(DateFormat.FULL),
				DateFormat.getDateTimeInstance(DateFormat.FULL, DateFormat.FULL),
			};

	private DateUtils()
	{
	}

	public static boolean isToday(Calendar date)
	{
		Calendar now=Calendar.getInstance();
		if (date.get(Calendar.DAY_OF_YEAR)!=now.get(Calendar.DAY_OF_YEAR)) return false;
		return date.get(Calendar.YEAR)==now.get(Calendar.YEAR);
	}

	public static boolean isWeekend(Calendar date)
	{
		int day=date.get(Calendar.DAY_OF_WEEK);
		return day==Calendar.SUNDAY || day==Calendar.SATURDAY;
	}

	public static Date parseDate(String value)
	{
		if ("sysdate".equalsIgnoreCase(value)) return new Date();
		for (int i=0; i<dateFormats.length; i++)
		{
			DateFormat dateFormat=dateFormats[i];
			try
			{
				return dateFormat.parse(value);
			}
			catch (ParseException e)
			{
				// Ignore
			}
		}
		return null;
	}

	/**
	 * Sets the date to the begin of the week.
	 */
	public static void setStartOfWeek(Calendar date)
	{
		date.add(Calendar.DATE,-getDayOfWeek(date));
		date.set(Calendar.HOUR_OF_DAY, 0);
		date.set(Calendar.MINUTE, 0);
		date.set(Calendar.SECOND, 0);
		date.set(Calendar.MILLISECOND, 0);
	}

	/**
	 * Returns the day of week depending if the week starts on sunday or monday.
	 *
	 * @return 0 for the first day of the week and 6 for the last.
	 */
	public static int getDayOfWeek(Calendar date)
	{
		int day=date.get(Calendar.DAY_OF_WEEK);
		if (getFirstDayOfWeek()==Calendar.SUNDAY) return day-1;
		else
		{
			if (day>=Calendar.MONDAY) return day-Calendar.MONDAY;
			else return 6;
		}
	}

	private static int firstDayOfWeek=Calendar.MONDAY;

	public static int getFirstDayOfWeek()
	{
		return firstDayOfWeek;
	}

	/**
	 * Sets the date to the end of the week.
	 */
	public static void setEndOfWeek(Calendar date)
	{
		date.add(Calendar.DATE,7-getDayOfWeek(date));
		date.set(Calendar.HOUR_OF_DAY, 0);
		date.set(Calendar.MINUTE, 0);
		date.set(Calendar.SECOND, 0);
		date.set(Calendar.MILLISECOND, 0);
	}

	public static Date add(Date date, int field, int value)
	{
		Calendar calendar=Calendar.getInstance();
		calendar.setTime(date);
		calendar.add(field, value);
		return calendar.getTime();
	}
}
