// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.utils;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

/**
 * @author Stefan Stiller
 * @version $Revision: 1.4 $, $Date: 2006/03/24 18:27:22 $
 */
public class DoubleKeyMap<K, J, V>
{
	private Map<K, Map<J, V>> map=new HashMap<K, Map<J, V>>();

	public DoubleKeyMap()
	{
	}

	public V get(K key1, J key2)
	{
		Map<J, V> nextMap=map.get(key1);
		if (nextMap==null) return null;
		return nextMap.get(key2);
	}

	public void put(K key1, J key2, V value)
	{
		Map<J, V> nextMap=map.get(key1);
		if (nextMap==null)
		{
			nextMap=new HashMap<J, V>();
			map.put(key1, nextMap);
		}
		nextMap.put(key2, value);
	}

	public Collection<V> get(K key1)
	{
		Map<J, V> nextMap=map.get(key1);
		if (nextMap==null) return Collections.emptySet();
		return nextMap.values();
	}

	public Collection<K> getKeys()
	{
		return map.keySet();
	}

	public Collection<J> getKeys(K key1)
	{
		Map<J, V> nextMap=map.get(key1);
		if (nextMap==null) return Collections.emptySet();
		return nextMap.keySet();
	}

	@SuppressWarnings({"SuspiciousMethodCalls"})
	public boolean containsKey(Object key1, Object key2)
	{
		Map<J, V> nextMap=map.get(key1);
		if (nextMap==null) return false;
		else return nextMap.containsKey(key2);
	}

	@SuppressWarnings({"SuspiciousMethodCalls"})
	public void remove(Object key1)
	{
		map.remove(key1);
	}

	@SuppressWarnings({"SuspiciousMethodCalls"})
	public void remove(Object key1, Object key2)
	{
		Map<J, V> nextMap=map.get(key1);
		if (nextMap!=null) nextMap.remove(key2);
	}

	public String toString()
	{
		return "DoubleKeyMap "+map;
	}

	public boolean isEmpty()
	{
		return map.isEmpty();
	}
}
