// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.utils.format;

import javax.swing.Icon;

/**
 * Interface class for displaying non boolean objects in UI components with 2 states.
 * I.e. 0 or '0' will be displayed as <code>false</code> and 1 or '1' will be displayed as
 * <code>true</code>.
 *
 * @author Stefan Stiller
 */
public class FlagFormat implements BooleanFormat
{
	private String name;

	public FlagFormat(String name)
	{
		this.name=name;
	}

	public String getName()
	{
		return name;
	}

	public String getGroup()
	{
		return null;
	}

	public boolean canFormat(Class aClass)
	{
		return aClass!=null && Integer.class==aClass || Long.class==aClass || Byte.class==aClass || Short.class==aClass || String.class==aClass;
	}

	public boolean canParse(Class aClass)
	{
		return canFormat(aClass);
	}

	public Boolean format(Object value)
	{
		if (value==null) return Boolean.FALSE;
		if (value instanceof Number) return ((Number)value).intValue()==1;
		if (value instanceof String) return "1".equals(value);
		return null;
	}

	public Object parse(Boolean value, Class targetClass)
	{
		boolean flag=Boolean.TRUE.equals(value);
		if (targetClass==String.class) return flag ? "1" : "0";
		if (targetClass==Byte.class) return flag ? (byte)1 : (byte)0;
		if (targetClass==Short.class) return flag ? (short)1 : (short)0;
		if (targetClass==Integer.class) return flag ? 1 : 0;
		if (targetClass==Long.class) return flag ? 1L : 0L;
		return Boolean.FALSE;
	}

	public Icon getIcon(Object value)
	{
		return null;
	}
}
