// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.utils.format;

import java.text.DateFormat;
import java.util.*;

import com.kiwisoft.utils.ClassFeatureManager;

/**
 * @author Stefan Stiller
 * @version $Revision: 1.5 $, $Date: 2006/03/24 18:58:21 $
 */
public class FormatManager extends ClassFeatureManager<ObjectFormat>
{
	private static FormatManager instance;

	public synchronized static FormatManager getInstance()
	{
		if (FormatManager.instance==null) FormatManager.instance=new FormatManager();
		return FormatManager.instance;
	}

	private FormatManager()
	{
		setFeature(Object.class, new DefaultObjectFormat(ObjectFormat.DEFAULT));
		setFeature(Number.class, new DefaultNumberFormat(ObjectFormat.DEFAULT));
		setFeature(Date.class, new DefaultDateFormat(ObjectFormat.DEFAULT));
		setFeature(Number.class, "Dual", new NonDecimalNumberFormat("Dual", 2, "b"));
		setFeature(Number.class, "Octal", new NonDecimalNumberFormat("Octal", 8, "o"));
		setFeature(Number.class, "Hexadecimal", new NonDecimalNumberFormat("Hexadecimal", 16, "h"));
		setFeature(Number.class, "Java", new JavaNumberFormat("Java"));
		setFeature(Number.class, "Money", new MoneyFormat("Money"));
		setFeature(String.class, "Bytes", new BytesFormat("Bytes"));
		setFeature(String.class, "Bytes (UTF-8)", new BytesFormat("Bytes (UTF-8)", "UTF-8"));
		setFeature(String.class, "Bytes (UTF-16)", new BytesFormat("Bytes (UTF-16)", "UTF-16"));
		setFeature(String.class, "Bytes (ISO-8859-1)", new BytesFormat("Bytes (ISO-8859-1)", "IS0-8859-1"));
		setFeature(Date.class, "Date only", new DefaultDateFormat("Date only", DateFormat.getDateInstance()));
		setFeature(Date.class, "Time only", new DefaultDateFormat("Time only", DateFormat.getTimeInstance()));
		setFeature(Date.class, "Date and Time", new DefaultDateFormat("Date and Time", DateFormat.getDateTimeInstance()));
		setFeature(Date.class, "Millis", new TimeMillisFormat("Millis"));
		FlagFormat flagFormat=new FlagFormat("Flag (0 or 1)");
		setFeature(Integer.class, "Flag (0 or 1)", flagFormat);
		setFeature(Long.class, "Flag (0 or 1)", flagFormat);
		setFeature(Short.class, "Flag (0 or 1)", flagFormat);
		setFeature(Byte.class, "Flag (0 or 1)", flagFormat);
		setFeature(String.class, "Flag (0 or 1)", flagFormat);
	}

	public ObjectFormat getFormat(Class aClass)
	{
		return getFeature(aClass);
	}

	public ObjectFormat getFormat(Class aClass, String variant)
	{
		return getFeature(aClass, variant);
	}

	public void setFormat(Class aClass, ObjectFormat format)
	{
		setFeature(aClass, format);
	}

	public ObjectFormat getRegisteredFormat(Class aClass)
	{
		return super.getRegisteredFeature(aClass);
	}

	public ObjectFormat getRegisteredFormat(Class aClass, String variant)
	{
		return super.getRegisteredFeature(aClass, variant);
	}
}
