// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.utils.format;

import java.math.BigDecimal;
import java.math.BigInteger;

import com.kiwisoft.utils.StringUtils;

/**
 * @author Stefan Stiller
 * @version $Revision: 1.3 $, $Date: 2006/03/24 18:33:39 $
 */
public class NonDecimalNumberFormat extends DefaultNumberFormat
{
	private String prefix;
	private int radix;

	public NonDecimalNumberFormat(String name, int radix, String prefix)
	{
		super(name);
		this.radix=radix;
		this.prefix=prefix;
	}

	public String format(Object value)
	{
		if (value instanceof Number)
		{
			Number number=(Number)value;
			if (number instanceof BigDecimal)
				return prefix+((BigDecimal)value).toBigInteger().toString(radix);
			else if (number instanceof BigInteger)
				return prefix+((BigInteger)value).toString(radix);
			else
				return prefix+Long.toString(number.longValue(), radix);
		}
		return super.format(value);
	}

	public boolean canParse(Class aClass)
	{
		return Number.class.isAssignableFrom(aClass);
	}

	public Object parse(String value, Class targetClass)
	{
		if (StringUtils.isEmpty(value)) return null;
		value=value.trim();
		Number number;
		if (value.startsWith(prefix)) value=value.substring(prefix.length());
		number=new BigInteger(value, radix);
		if (targetClass.isInstance(number)) return number;
		if (targetClass==Double.class) return new Double(number.doubleValue());
		if (targetClass==Float.class) return new Float(number.floatValue());
		if (targetClass==Long.class) return new Long(number.longValue());
		if (targetClass==Integer.class) return new Integer(number.intValue());
		if (targetClass==Short.class) return new Short(number.shortValue());
		if (targetClass==Byte.class) return new Byte(number.byteValue());
		if (targetClass==BigDecimal.class) return new BigDecimal((BigInteger)number);
		if (targetClass==BigInteger.class) return number;
		return number;
	}
}
