// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.utils.format;

import javax.swing.Icon;

/**
 * Interface class for displaying objects in UI text components like table cells, lists,
 * labels and text fields. Implementations must not directly implement this interface but
 * have to implement on of the child interfaces {@link TextFormat} or {@link BooleanFormat}. 
 *
 * @author Stefan Stiller
 * @version $Revision: 1.5 $, $Date: 2006/03/24 18:33:39 $
 */
public interface ObjectFormat
{
	/**
	 * Used for default formats.
	 * @see #getName()
	 */
	public static final String DEFAULT="Default";

	/**
	 * Returns the name of the format. Return "Default" if the format should be the default format
	 * for the handled classes (works only if no default format has already been installed).
	 * @return The name of the format. Must not be <code>null</code>.
	 */
	String getName();

	/**
	 * The name of the group the format belongs to. The group defines the sub menu under which
	 * the format will appear in the context menu.
	 *
	 * @return The name of the format group. If <code>null</code> the format will appear in the root menu.
	 */
	String getGroup();

	/**
	 * @return <code>true</code> if the format can format objects of the specified class.
	 */
	boolean canFormat(Class aClass);

	/**
	 * Returns the <code>Boolean</code> representing the specified value.
	 *
	 * @param value The object which should be formated.
	 * @return The formatted object.
	 * @see #canFormat(Class)
	 */
	Object format(Object value);

	Icon getIcon(Object value);

	/**
	 * @return <code>true</code> if the format can create objects of the specified class
	 * from <code>String</code> objects.
	 */
	boolean canParse(Class aClass);


}
