// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.utils.gui.adapters;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

import org.apache.commons.beanutils.PropertyUtils;

import com.kiwisoft.utils.ObservableBean;
import com.kiwisoft.utils.gui.lookup.LookupField;
import com.kiwisoft.utils.gui.lookup.LookupSelectionListener;
import com.kiwisoft.utils.gui.lookup.LookupEvent;

/**
 * An adapter binding a lookup field to a bean property. 
 *
 * @author Stefan Stiller
 */
@SuppressWarnings({"unchecked"})
public class LookupAdapter implements PropertyChangeListener, LookupSelectionListener, ComponentAdapter
{
	private LookupField lookupField;
	private ObservableBean object;
	private String property;

	public LookupAdapter(LookupField lookupField)
	{
		this.lookupField=lookupField;
	}

	public void bind(ObservableBean object, String property)
	{
		this.object=object;
		this.property=property;
		boolean readable=false;
		boolean writeable=false;
		lookupField.setValue(null);
		if (object!=null)
		{
			readable=PropertyUtils.isReadable(object, property);
			try
			{
				writeable=readable && PropertyUtils.isWriteable(object, property);
			}
			catch (Exception e)
			{
				e.printStackTrace();
			}
			if (readable)
			{
				updateComponent();
				object.addPropertyChangeListener(property, this);
			}
			if (writeable)
			{
				lookupField.addSelectionListener(this);
			}
		}
		lookupField.setEnabled(readable);
	}

	private void updateComponent()
	{
		try
		{
			Object value=PropertyUtils.getProperty(object, property);
			lookupField.setValue(value);
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	public void unbind()
	{
		if (object!=null) object.removePropertyChangeListener(property, this);
		lookupField.removeSelectionListener(this);
	}

	public void propertyChange(PropertyChangeEvent evt)
	{
		updateComponent();
	}

	public void selectionChanged(LookupEvent event)
	{
		if (object!=null && lookupField.isEnabled())
		{
			try
			{
				PropertyUtils.setProperty(object, property, lookupField.getValue());
			}
			catch (Exception e1)
			{
				e1.printStackTrace();
			}
		}
	}

	public boolean stopEditing()
	{
		return lookupField.canTraverseFocus();
	}
}
