// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.utils.gui.adapters;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import java.awt.event.FocusListener;
import java.awt.event.FocusEvent;
import javax.swing.text.JTextComponent;

import org.apache.commons.beanutils.PropertyUtils;

import com.kiwisoft.utils.ObservableBean;

/**
 * An adapter binding a text field to a String property. 
 *
 * @author Stefan Stiller
 */
public class TextComponentAdapter implements PropertyChangeListener, FocusListener, ComponentAdapter
{
	private JTextComponent textComponent;
	private ObservableBean object;
	private String property;

	public TextComponentAdapter(JTextComponent textComponent)
	{
		this.textComponent=textComponent;
	}

	public void bind(ObservableBean object, String property)
	{
		this.object=object;
		this.property=property;
		boolean readable=false;
		boolean writeable=false;
		textComponent.setText("");
		if (object!=null)
		{
			readable=PropertyUtils.isReadable(object, property);
			try
			{
				writeable=readable && PropertyUtils.isWriteable(object, property);
			}
			catch (Exception e)
			{
				e.printStackTrace();
			}
			if (readable)
			{
				updateComponent();
				object.addPropertyChangeListener(property, this);
			}
			if (writeable)
			{
				textComponent.addFocusListener(this);
			}
		}
		textComponent.setEnabled(readable);
		textComponent.setEditable(writeable);
	}

	private void updateComponent()
	{
		try
		{
			Object value=PropertyUtils.getProperty(object, property);
			textComponent.setText(value!=null ? value.toString() : "");
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	public void unbind()
	{
		if (object!=null) object.removePropertyChangeListener(property, this);
		textComponent.removeFocusListener(this);
	}

	public void propertyChange(PropertyChangeEvent evt)
	{
		updateComponent();
	}

	public void focusGained(FocusEvent e)
	{
	}

	public void focusLost(FocusEvent e)
	{
		if (object!=null && textComponent.isEnabled())
		{
			try
			{
				PropertyUtils.setProperty(object, property, textComponent.getText());
			}
			catch (Exception e1)
			{
				e1.printStackTrace();
			}
		}
	}

	public boolean stopEditing()
	{
		focusLost(null);
		return true;
	}
}
