// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.utils.gui.table;

import java.util.List;
import java.awt.*;
import javax.swing.*;
import javax.swing.table.DefaultTableCellRenderer;

import com.kiwisoft.utils.format.TextFormat;
import com.kiwisoft.utils.gui.ComponentDecorator;

/**
 * @author Stefan Stiller
 * @version $Revision: 1.5 $, $Date: 2006/03/24 18:58:20 $
 */
public class DefaultObjectRenderer extends DefaultTableCellRenderer implements ObjectRenderer
{
	private TextFormat format;
	private Font font;
	private List<ComponentDecorator> decorators;
	private Color rowBackground;
	private Color cellBackground;
	private Color cellForeground;
	private Color rowForeground;

	public DefaultObjectRenderer(TextFormat format)
	{
		this.format=format;
	}

	public void setCellForeground(Color foreground)
	{
		cellForeground=foreground;
	}

	public void setRowForeground(Color foreground)
	{
		rowForeground=foreground;
	}

	public void setCellBackground(Color background)
	{
		cellBackground=background;
	}

	public void setRowBackground(Color background)
	{
		rowBackground=background;
	}

	public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
	{
		TableRendererFactory.initializeRowColor(table, this, row);
		if (SortableTable.TOP.equals(table.getClientProperty(SortableTable.VERTICAL_CELL_ALIGNMENT)))
			setVerticalAlignment(SwingConstants.TOP);
		setHorizontalAlignment(format.getHorizontalAlignment(value));
		String text=format.format(value);
		if (text==null) text=(String)table.getClientProperty(SortableTable.NULL_STRING);
		Icon icon=format.getIcon(value);
		if (icon!=null) setIcon(icon);
		else setIcon(null);
		if (cellBackground!=null) setBackground(cellBackground);
		else if (rowBackground!=null) setBackground(rowBackground);
		else setBackground(table.getBackground());
		if (cellForeground!=null) setForeground(cellForeground);
		else if (rowForeground!=null) setForeground(rowForeground);
		else setForeground(table.getForeground());
		Component rendererComponent=super.getTableCellRendererComponent(table, text, isSelected, hasFocus, row, column);
		if (font!=null) rendererComponent.setFont(font);
		return rendererComponent;
	}

	protected void paintComponent(Graphics g)
	{
		TableRendererFactory.paintBackground(g, this, cellBackground, rowBackground);
		TableRendererFactory.prepareComponent(g, this, decorators);
		super.paintComponent(g);
		TableRendererFactory.decorateComponent(g, this, decorators);
	}

	public void setDecorators(List<ComponentDecorator> decorators)
	{
		this.decorators=decorators;
	}

	public void setTextFont(Font font)
	{
		this.font=font;
	}

	public ObjectRenderer cloneRenderer()
	{
		return new DefaultObjectRenderer(format);
	}

	public String toString()
	{
		return "DefaultObjectRenderer[format="+format+"]";
	}
}
