// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.utils.gui.table;

import java.util.Map;

import com.kiwisoft.utils.StringUtils;

/**
 * Created by IntelliJ IDEA.
 * User: Stefan1
 * Date: 04.11.2006
 * Time: 11:07:50
 * To change this template use File | Settings | File Templates.
 */
public class PropertiesTableModel extends DefaultSortableTableModel<String>
{
	private static final String[] COLUMNS={"name", "value"};
	private boolean modified;

	public PropertiesTableModel(Map<String, String> data)
	{
		createData(data);
	}

	public int getColumnCount()
	{
		return COLUMNS.length;
	}

	public String getColumnName(int column)
	{
		return COLUMNS[column];
	}

//	public void setDatabase(DatabaseModel database)
//	{
//		if (this.database!=database)
//		{
//			clear();
//			this.database=database;
//			createData();
//		}
//	}

	private void createData(Map<String, String> data)
	{
		if (data!=null)
		{
			for (Map.Entry<String,String> entry : data.entrySet())
			{
				addRow(new Row(entry.getKey(), entry.getValue()));
			}
			modified=false;
			sort();
		}
	}

	public void createRow()
	{
		addRow(new Row(null, null));
	}

	public boolean isModified()
	{
		return modified;
	}

	public class Row extends SortableTableRow<String>
	{
		private String value;

		private Row(String key, String value)
		{
			super(key);
			this.value=value;
		}

		public Object getDisplayValue(int column)
		{
			switch (column)
			{
				case 0: return getUserObject();
				case 1: return value;
			}
			return null;
		}

		public boolean isEditable(int column)
		{
			switch (column)
			{
				case 0: return true;
				case 1: return !StringUtils.isEmpty(getUserObject());
			}
			return false;
		}

		public int setValue(Object value, int column)
		{
			if (value instanceof String || value==null)
			{
				switch (column)
				{
					case 0:
						setUserObject((String)value);
						modified=true;
						return TableConstants.CELL_UPDATE;
					case 1:
						this.value=(String)value;
						modified=true;
						return TableConstants.CELL_UPDATE;
				}
			}
			return super.setValue(value, column);
		}

		public String getValue()
		{
			return value;
		}
	}
}
