// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.utils.gui.table;

import java.awt.Color;
import java.util.*;

import javax.swing.SwingUtilities;

import com.kiwisoft.utils.gui.ObjectStyle;
import com.kiwisoft.utils.StringUtils;
import com.kiwisoft.utils.Utils;

/**
 * @author Stefan Stiller
 * @version $Revision: $, $Date: $
 */
@SuppressWarnings({"unchecked"})
public class SetTableModel extends DefaultSortableTableModel
{
	private Class type;
	private SetTable table;
	private boolean editable;

	public SetTableModel(Class type)
	{
		super("value");
		setResortable(true);
		this.type=type;
		//noinspection unchecked
		addRow(new InputRow());
	}

	public Class getCellClass(int row, int col)
	{
		return type;
	}

	public void setTable(SetTable table)
	{
		this.table=table;
	}

	public void setEditable(boolean editable)
	{
		this.editable=editable;
	}

	public void setSet(Set set)
	{
		Set oldSet=getSet();
		if (!Utils.equals(oldSet, set))
		{
			List<SortableTableRow> rows=new ArrayList<SortableTableRow>(set!=null ? set.size()+1 : 1);
			if (set!=null) for (Object o : set) rows.add(new Row(o));
			rows.add(new InputRow());
			setRows(rows);
		}
	}

	public Set getSet()
	{
		Set set=new HashSet();
		for (Iterator it=rows.iterator(); it.hasNext();)
		{
			SortableTableRow row=(SortableTableRow)it.next();
			if (row instanceof Row) set.add(row.getUserObject());
		}
		return set;
	}

	class Row extends SortableTableRow<Object>
	{
		public Row(Object userObject)
		{
			super(userObject);
		}

		public Object getDisplayValue(int column)
		{
			return getUserObject();
		}

		public int setValue(Object value, int col)
		{
			if (value==null || (value instanceof String && StringUtils.isEmpty((String)value)))
			{
				int index=indexOfRow(this);
				removeRow(this);
				if (index<getRowCount())
				{
					table.getSelectionModel().setSelectionInterval(index, index);
				}
				return NO_UPDATE;
			}
			else if (type.isInstance(value))
			{
				for (Iterator it=rows.iterator(); it.hasNext();)
				{
					SortableTableRow row=(SortableTableRow)it.next();
					if (row!=this && row instanceof Row && value.equals(row.getUserObject()))
						return NO_UPDATE;
				}
				setUserObject(value);
				return CELL_UPDATE;
			}
			return NO_UPDATE;
		}

		public boolean isEditable(int column)
		{
			return editable;
		}
	}

	private class InputRow extends SortableTableRow<Object>
	{
		private ObjectStyle style;

		protected InputRow()
		{
			super(null);
			style=new ObjectStyle(null, new Color(220, 220, 255));
		}

		public Object getDisplayValue(int column)
		{
			return null;
		}

		protected int getSortPriority()
		{
			return 1;
		}

		public boolean isEditable(int column)
		{
			return editable;
		}

		public ObjectStyle getCellStyle(int col)
		{
			return style;
		}

		public int setValue(final Object value, int col)
		{
			if (value!=null && type.isInstance(value))
			{
				for (Iterator it=rows.iterator(); it.hasNext();)
				{
					SortableTableRow row=(SortableTableRow)it.next();
					if (row instanceof Row && value.equals(row.getUserObject())) return NO_UPDATE;
				}
				SwingUtilities.invokeLater(new Runnable()
				{
					public void run()
					{
						//noinspection unchecked
						int index=indexOfRow(InputRow.this);
						addRow(new Row(value), index);
						index=indexOfRow(InputRow.this);
						table.getSelectionModel().setSelectionInterval(index, index);
					}
				});
			}
			return NO_UPDATE;
		}
	}
}