// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.utils.gui.table;

import java.awt.event.FocusEvent;
import java.awt.event.KeyEvent;
import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;
import javax.swing.JButton;
import javax.swing.JTextField;
import javax.swing.KeyStroke;

import com.kiwisoft.utils.gui.lookup.LookupField;
import com.kiwisoft.utils.gui.lookup.Lookup;

/**
 * @author Stefan Stiller
 * @version $Revision: 1.2 $, $Date: 2006/03/24 18:04:07 $
 */
class TableLookupField<T> extends LookupField<T>
{
    private List<JButton> buttons;

    public TableLookupField(Lookup<T> lookup)
    {
        super(new TableTextField(), lookup, null);
    }

    protected String getLookupIcon()
    {
        return "lookup.table";
    }

    public void focusLost(FocusEvent e)
    {
    }

    protected boolean processKeyBinding(KeyStroke ks, KeyEvent e, int condition, boolean pressed)
    {
        TableTextField tableTextField=(TableTextField)getTextField();
        if (!tableTextField.isFocusOwner())
        {
            tableTextField.requestFocus();
            return tableTextField.processKeyBinding(ks, e, condition, pressed);
        }
        return super.processKeyBinding(ks, e, condition, pressed);
    }

    protected void layoutButtons(JTextField textField, JButton btnEdit, JButton btnCreate, JButton btnLookup)
    {
        buttons=new ArrayList<JButton>();
        setLayout(null);
        add(textField);
        if (btnEdit!=null)
        {
            add(btnEdit);
            buttons.add(btnEdit);
        }
        if (btnCreate!=null)
        {
            add(btnCreate);
            buttons.add(btnCreate);
        }
        if (btnLookup!=null)
        {
            add(btnLookup);
            buttons.add(btnLookup);
        }
    }

    public void setBounds(int x, int y, int width, int height)
    {
        super.setBounds(x, y, width, height);
        int buttonHeight=Math.min(20, height);
        int xPos=width-buttonHeight*buttons.size();
        getTextField().setBounds(0, 0, xPos, height);
        for (Iterator it=buttons.iterator(); it.hasNext();)
        {
            JButton button=(JButton)it.next();
            button.setBounds(xPos, 0, buttonHeight, buttonHeight);
            xPos+=buttonHeight;
        }
    }
}
