// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.utils.gui.tree;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.Graphics;
import java.util.List;
import javax.swing.JTree;
import javax.swing.UIManager;
import javax.swing.tree.DefaultTreeCellRenderer;

import com.kiwisoft.utils.format.ObjectFormat;
import com.kiwisoft.utils.format.FormatManager;
import com.kiwisoft.utils.gui.ObjectStyle;
import com.kiwisoft.utils.gui.ComponentDecorator;
import com.kiwisoft.utils.gui.table.TableRendererFactory;

public class GenericTreeNodeRenderer extends DefaultTreeCellRenderer
{
	private List<ComponentDecorator> decorators;

	public GenericTreeNodeRenderer()
	{
	}

	public Component getTreeCellRendererComponent(JTree tree, Object value, boolean selected, boolean expanded, boolean leaf, int row, boolean hasFocus)
	{
		if (value instanceof GenericTreeNode)
		{
			GenericTreeNode node=(GenericTreeNode)value;
			Object userObject=node.getUserObject();
			if (userObject!=null)
			{
				ObjectFormat format=FormatManager.getInstance().getFormat(userObject.getClass());
				setText(String.valueOf(format.format(userObject)));
				setIcon(format.getIcon(userObject));
			}
			else
			{
				setText(node.getText());
				setIcon(node.getIcon());
			}
			ObjectStyle style=node.getStyle();
			this.selected=selected;
			Color foreground=style!=null ? style.getForeground() : null;
			if (!selected)
			{
				if (foreground==null) foreground=getTextNonSelectionColor();
				setForeground(foreground);
				Color background=style!=null ? style.getBackground() : null;
				if (background==null) background=tree.getBackground();
				setBackgroundNonSelectionColor(background);
			}
			else
			{
				if (foreground==null) foreground=getTextSelectionColor();
				setForeground(foreground);
				setBackgroundSelectionColor((Color)UIManager.get("Tree.selectionBackground"));
			}
			if (style!=null)
			{
				decorators=style.getDecorators();
			}
			Font font=node.getFont(tree);
			setFont(font!=null ? font : tree.getFont());
		}
		return this;
	}

	protected void paintComponent(Graphics g)
	{
		TableRendererFactory.prepareComponent(g, this, decorators);
		super.paintComponent(g);
		TableRendererFactory.decorateComponent(g, this, decorators);
	}

}
