// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.utils.xml;

/**
 * Interface classes have to implement if the <code>{@link XMLHandler} is be used to automatically
 * create objects from tags.
 *
 * @author Stefan Stiller
 * @version $Revision: 1.3 $, $Date: 2006/03/24 17:44:16 $
 * @see XMLHandler
 */
public interface XMLObject
{
	/**
	 * Method called by the XMLHandler to set the attributes
	 * of an element.
	 *
	 * @param name The name of the attribute.
	 * @param value The value of the attribute.
	 */
	public void setXMLAttribute(XMLContext context, String name, String value);

	public void setXMLReference(XMLContext context, String name, Object value);

	/**
	 * Method called by the XMLHandler to set the content of
	 * an element.
	 *
	 * @param value The content of the element.
	 */
	public void setXMLContent(XMLContext context, String value);

	/**
	 * Method called by the XMLHandler to add child elements
	 * to an element.
	 *
	 * @param element The new child element.
	 */
	public void addXMLElement(XMLContext context, XMLObject element);
}
